<?php

namespace Drupal\commerce_checkout_url\Access;

use Drupal\commerce_checkout_url\SecurityKeyInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Session\AccountInterface;
use Psr\Container\ContainerInterface;

/**
 * Checks access for checkout url claim form.
 *
 * This class is designed to be used with '_custom_access' route requirement.
 *
 * @see \Drupal\Core\Access\CustomAccessCheck
 */
class ClaimAccessCheck implements ContainerInjectionInterface {

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_checkout_url.security_key')
    );
  }

  /**
   * Constructs a new access check for checkout url claim form.
   *
   * @param \Drupal\commerce_checkout_url\SecurityKeyInterface $securityKey
   *   The security key service.
   */
  public function __construct(
    protected SecurityKeyInterface $securityKey
  ) {
  }

  /**
   * Checks if the user is user 1 and grants access if so.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $commerce_order
   *   The commerce order.
   * @param string $expire
   *   A hex-encoded UNIX timestamp.
   * @param string $hmac
   *   The message authentication code.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user account.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function checkAccess(
    OrderInterface $commerce_order,
    string $expire,
    string $hmac,
    AccountInterface $account
  ) {
    $customer = $commerce_order->getCustomer();
    $orderAccessible =
      $customer->isAnonymous() ||
      $customer->id() === $account->id();
    $hmacValid = $this->securityKey->verify($commerce_order->id(), $expire, $hmac);

    return AccessResultAllowed::allowedIf($orderAccessible && $hmacValid)->mergeCacheMaxAge(0);
  }

}
