<?php

namespace Drupal\commerce_checkout_url;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Site\Settings;

/**
 * Provides methods to generate and check the HMAC for commerce checkout urls.
 */
class SecurityKey implements SecurityKeyInterface {

  /**
   * Constructs a download security key service.
   *
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The request time service.
   * @param string $hmacKey
   *   The HMAC key. Defaults commerce_checkout_url_private_key setting and
   *   falls back to Drupal private key + hash salt
   */
  public function __construct(
    protected TimeInterface $time,
    protected $hmacKey = NULL
  ) {
    if (!isset($this->hmacKey)) {
      $this->hmacKey = Settings::get('commerce_checkout_url_private_key', \Drupal::service('private_key')->get() . Settings::getHashSalt());
    }
  }

  public function verify(int $orderId, string $expire, string $hmac) {
    $result = FALSE;

    if (strlen($expire) && strlen($hmac)) {
      $calculated_hmac = Crypt::hmacBase64($orderId . $expire, $this->hmacKey);
      $result = hash_equals($calculated_hmac, $hmac) &&
        $this->time->getRequestTime() <= hexdec($expire);
    }

    return $result;
  }

  public function generate(int $orderId, string $expire) {
    return Crypt::hmacBase64($orderId . $expire, $this->hmacKey);
  }

}
