<?php

namespace Drupal\Tests\commerce_checkout_url\Functional;

use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\commerce_store\StoreCreationTrait;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\commerce\Traits\CommerceBrowserTestTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\Entity\Role;

/**
 * Test commerce checkout url claim form.
 *
 * @group commerce_checkout_url
 */
class AccessTest extends BrowserTestBase {
  use StoreCreationTrait;
  use CommerceBrowserTestTrait;
  use UserCreationTrait;

  /**
   * The store entity.
   *
   * @var \Drupal\commerce_store\Entity\Store
   */
  protected $store;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'field',
    'commerce',
    'commerce_cart',
    'commerce_order',
    'commerce_price',
    'commerce_product',
    'commerce_store',
    'commerce_checkout',
    'commerce_checkout_url',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  protected function setUp(): void {
    parent::setUp();

    $this->store = $this->createStore();
  }

  /**
   * Creates a dummy product for use with other tests.
   */
  public function createDummyOrder($uid = 0): OrderInterface {
    $variation = ProductVariation::create([
      'type' => 'default',
      'sku' => strtolower($this->randomMachineName()),
      'price' => [
        'number' => 9.99,
        'currency_code' => 'USD',
      ],
    ]);
    $variation->save();

    $product = Product::create([
      'type' => 'default',
      'title' => 'My product',
      'variations' => [$variation],
      'stores' => [$this->store],
    ]);
    $product->save();

    $orderItem = OrderItem::create([
      'type' => 'default',
      'purchased_entity' => $variation,
    ]);
    $orderItem->save();

    $order = Order::create([
      'type' => 'default',
      'state' => 'draft',
      'order_items' => [$orderItem],
      'store_id' => $this->store->id(),
      'uid' => $uid,
      'cart' => TRUE,
    ]);
    $order->save();

    return $order;
  }

  /**
   * Tests anonymous user accessing an anonymous order.
   */
  public function testAnonymousCheckout() {
    Role::load(Role::ANONYMOUS_ID)
      ->grantPermission('access checkout')
      ->save();

    $order = $this->createDummyOrder(0);

    // Attempt to access an anonymous order directly.
    $this->drupalGet("checkout/{$order->id()}");
    $this->assertSession()->statusCodeEquals(403, 'Access to cart is denied.');

    $this->drupalGet("checkout-resume/{$order->id()}");
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    /** @var \Drupal\commerce_checkout_url\LinkGeneratorInterface */
    $linkGenerator = \Drupal::service('commerce_checkout_url.link_generator');
    $url = $linkGenerator->generate($order);
    $this->assertNotNull($url);
    $this->drupalGet($url->toString());

    // Check response.
    $this->assertSession()->statusCodeEquals(200, 'Access to claim form is granted with HMAC protected URL.');
    $this->assertSession()->responseContains(new FormattableMarkup('This is a one-time link for %label. The link will expire on', [
      '%label' => $order->label(),
    ]));
    $this->assertSession()->pageTextContains('Click on this button to proceed to checkout.');

    // Submit the form.
    $this->submitForm([], 'Checkout');
    $this->assertSession()->statusCodeEquals(200, 'Access to checkout is granted.');
    $this->assertStringEndsWith("checkout/{$order->id()}/login", $this->getSession()->getCurrentUrl());
  }

  /**
   * Tests authenticated user accessing an anonymous order.
   */
  public function testAuthenticatedCheckout() {
    $siteUser = $this->setUpCurrentUser(permissions: ['access checkout']);
    $this->drupalLogin($siteUser);

    $order = $this->createDummyOrder(0);

    // Attempt to access an anonymous order directly.
    $this->drupalGet("checkout/{$order->id()}");
    $this->assertSession()->statusCodeEquals(403, 'Access to cart is denied.');

    $this->drupalGet("checkout-resume/{$order->id()}");
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    /** @var \Drupal\commerce_checkout_url\LinkGeneratorInterface */
    $linkGenerator = \Drupal::service('commerce_checkout_url.link_generator');
    $url = $linkGenerator->generate($order);
    $this->assertNotNull($url);
    $this->drupalGet($url->toString());

    // Check response.
    $this->assertSession()->statusCodeEquals(200, 'Access to claim form is granted with HMAC protected URL.');
    $this->assertSession()->responseContains(new FormattableMarkup('This is a one-time link for %label. The link will expire on', [
      '%label' => $order->label(),
    ]));
    $this->assertSession()->pageTextContains('Click on this button to proceed to checkout.');

    // Submit the form.
    $this->submitForm([], 'Checkout');
    $this->assertSession()->statusCodeEquals(200, 'Access to checkout is granted.');
    $this->assertStringEndsWith("checkout/{$order->id()}/order_information", $this->getSession()->getCurrentUrl());
  }

  /**
   * Tests authenticated user accessing its own order.
   */
  public function testOwnOrderCheckout() {
    $orderOwner = $this->setUpCurrentUser(permissions: ['access checkout']);
    $this->drupalLogin($orderOwner);

    $order = $this->createDummyOrder($orderOwner->id());

    // Attempt to access a non-anonymous order directly.
    $this->drupalGet("checkout/{$order->id()}");
    $this->assertSession()->statusCodeEquals(200, 'Access to own cart is granted.');

    $this->drupalGet("checkout-resume/{$order->id()}");
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    /** @var \Drupal\commerce_checkout_url\LinkGeneratorInterface */
    $linkGenerator = \Drupal::service('commerce_checkout_url.link_generator');
    $url = $linkGenerator->generate($order);
    $this->assertNotNull($url);
    $this->drupalGet($url->toString());

    // Check response.
    $this->assertSession()->statusCodeEquals(200, 'Access to claim form is granted with HMAC protected URL.');
    $this->assertSession()->responseContains(new FormattableMarkup('This is a one-time link for %label. The link will expire on', [
      '%label' => $order->label(),
    ]));
    $this->assertSession()->pageTextContains('Click on this button to proceed to checkout.');

    // Submit the form.
    $this->submitForm([], 'Checkout');
    $this->assertSession()->statusCodeEquals(200, 'Access to checkout is granted.');
    $this->assertStringEndsWith("checkout/{$order->id()}/order_information", $this->getSession()->getCurrentUrl());
  }

  /**
   * Tests anonymous user attempting to access an order already owned.
   */
  public function testOwnedOrderCheckout() {
    Role::load(Role::ANONYMOUS_ID)
      ->grantPermission('access checkout')
      ->save();

    $orderOwner = $this->setUpCurrentUser(permissions: ['access checkout']);
    $order = $this->createDummyOrder($orderOwner->id());

    // Attempt to access an anonymous order directly.
    $this->drupalGet("checkout/{$order->id()}");
    $this->assertSession()->statusCodeEquals(403, 'Access to cart is denied.');

    $this->drupalGet("checkout-resume/{$order->id()}");
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    /** @var \Drupal\commerce_checkout_url\LinkGeneratorInterface */
    $linkGenerator = \Drupal::service('commerce_checkout_url.link_generator');
    $url = $linkGenerator->generate($order);
    $this->assertNotNull($url);
    $this->drupalGet($url->toString());

    // Check response.
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied even with HMAC protected URL.');

    // Retry with authenticated user.
    $siteUser = $this->setUpCurrentUser(permissions: ['access checkout']);
    $this->drupalLogin($siteUser);

    // Attempt to access an owned order directly.
    $this->drupalGet("checkout/{$order->id()}");
    $this->assertSession()->statusCodeEquals(403, 'Access to cart is denied.');

    $this->drupalGet("checkout-resume/{$order->id()}");
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied without HMAC.');

    // Generate a HMAC protected URL and use that.
    /** @var \Drupal\commerce_checkout_url\LinkGeneratorInterface */
    $linkGenerator = \Drupal::service('commerce_checkout_url.link_generator');
    $url = $linkGenerator->generate($order);
    $this->assertNotNull($url);
    $this->drupalGet($url->toString());

    // Check response.
    $this->assertSession()->statusCodeEquals(404, 'Access to claim form is denied even with HMAC protected URL.');
  }

}
